{% extends "base.sh.j2" %}

{# All Available Blocks #}

{# before build #}
{# {% block install_commands %} #}
{# {% endblock install_commands %} #}

{# build body #}
{# {% block build_commands %} #}
{# {% endblock %} #}

{# post build #}
{# {% block finally_build_commands %} #}
{# {% endblock %} #}